@@ -32,7 +32,6 @@
 #include <gtk/gtk.h>
 
 #include <telepathy-glib/util.h>
-#include <libmissioncontrol/mc-account.h>
 
 #include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-call-factory.h>
@@ -121,7 +120,7 @@ contact_list_view_tooltip_destroy_cb (GtkWidget              *widget,
 				      EmpathyContactListView *view)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
-	
+
 	if (priv->tooltip_widget) {
 		DEBUG ("Tooltip destroyed");
 		g_object_unref (priv->tooltip_widget);
@@ -230,7 +229,7 @@ contact_list_view_drag_got_contact (EmpathyTpContactFactory *factory,
 	if (data->new_group) {
 		empathy_contact_list_add_to_group (list, contact, data->new_group);
 	}
-	if (data->old_group && data->action == GDK_ACTION_MOVE) {	
+	if (data->old_group && data->action == GDK_ACTION_MOVE) {
 		empathy_contact_list_remove_from_group (list, contact, data->old_group);
 	}
 }
@@ -247,13 +246,13 @@ contact_list_view_drag_data_received (GtkWidget         *view,
 	EmpathyContactListViewPriv *priv;
 	EmpathyAccountManager      *account_manager;
 	EmpathyTpContactFactory    *factory = NULL;
-	McAccount                  *account;
+	EmpathyAccount             *account = NULL;
 	GtkTreeModel               *model;
 	GtkTreeViewDropPosition     position;
 	GtkTreePath                *path;
 	const gchar                *id;
 	gchar                     **strv = NULL;
-	const gchar                *account_id;
+	const gchar                *account_id = NULL;
 	const gchar                *contact_id;
 	gchar                      *new_group = NULL;
 	gchar                      *old_group = NULL;
@@ -293,28 +292,28 @@ contact_list_view_drag_data_received (GtkWidget         *view,
 		goto OUT;
 	}
 
-	id = (const gchar*) selection->data;
+	id = (const gchar*) gtk_selection_data_get_data (selection);
 	DEBUG ("Received %s%s drag & drop contact from roster with id:'%s'",
 		context->action == GDK_ACTION_MOVE ? "move" : "",
 		context->action == GDK_ACTION_COPY ? "copy" : "",
 		id);
 
-	strv = g_strsplit (id, "/", 2);
-	account_id = strv[0];
-	contact_id = strv[1];
-	account = mc_account_lookup (account_id);
+	account_manager = empathy_account_manager_dup_singleton ();
+	strv = g_strsplit (id, ":", 2);
+	if (g_strv_length (strv) == 2) {
+		account_id = strv[0];
+		contact_id = strv[1];
+		account = empathy_account_manager_get_account (account_manager, account_id);
+	}
 	if (account) {
 		TpConnection *connection;
 
-		/* FIXME: We assume we have already an account manager */
-		account_manager = empathy_account_manager_dup_singleton ();
-		connection = empathy_account_manager_get_connection (account_manager,
-								     account);
+		connection = empathy_account_get_connection (account);
 		if (connection) {
 			factory = empathy_tp_contact_factory_dup_singleton (connection);
 		}
-		g_object_unref (account_manager);
 	}
+	g_object_unref (account_manager);
 
 	if (!factory) {
 		DEBUG ("Failed to get factory for account '%s'", account_id);
@@ -367,6 +366,7 @@ contact_list_view_drag_motion (GtkWidget      *widget,
 	gboolean               is_row;
 	gboolean               is_different = FALSE;
 	gboolean               cleanup = TRUE;
+	int                    action = 0;
 
 	is_row = gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
 						x,
@@ -385,6 +385,13 @@ contact_list_view_drag_motion (GtkWidget      *widget,
 		cleanup &= FALSE;
 	}
 
+	if (context->actions == GDK_ACTION_COPY) {
+		action = context->suggested_action;
+	} else if (context->actions & GDK_ACTION_MOVE) {
+		action = GDK_ACTION_MOVE;
+	}
+	gdk_drag_status (context, action, time);
+
 	if (!is_different && !cleanup) {
 		return TRUE;
 	}
@@ -451,7 +458,7 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 	GtkTreeIter                 iter;
 	GtkTreeModel               *model;
 	EmpathyContact             *contact;
-	McAccount                  *account;
+	EmpathyAccount             *account;
 	const gchar                *contact_id;
 	const gchar                *account_id;
 	gchar                      *str;
@@ -481,10 +488,10 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 	}
 
 	account = empathy_contact_get_account (contact);
-	account_id = mc_account_get_unique_name (account);
+	account_id = empathy_account_get_unique_name (account);
 	contact_id = empathy_contact_get_id (contact);
 	g_object_unref (contact);
-	str = g_strconcat (account_id, "/", contact_id, NULL);
+	str = g_strconcat (account_id, ":", contact_id, NULL);
 
 	switch (info) {
 	case DND_DRAG_TYPE_CONTACT_ID:
@@ -820,16 +827,20 @@ contact_list_view_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean is_group;
 	gboolean is_active;
 	gboolean show_status;
+	gchar *name;
 
 	gtk_tree_model_get (model, iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
 			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, &show_status,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	g_object_set (cell,
 		      "show-status", show_status,
+		      "text", name,
 		      NULL);
+	g_free (name);
 
 	contact_list_view_cell_set_background (view, cell, is_group, is_active);
 }
@@ -854,7 +865,7 @@ contact_list_view_expander_cell_data_func (GtkTreeViewColumn     *column,
 		gboolean     row_expanded;
 
 		path = gtk_tree_model_get_path (model, iter);
-		row_expanded = gtk_tree_view_row_expanded (GTK_TREE_VIEW (column->tree_view), path);
+		row_expanded = gtk_tree_view_row_expanded (GTK_TREE_VIEW (gtk_tree_view_column_get_tree_view (column)), path);
 		gtk_tree_path_free (path);
 
 		g_object_set (cell,
@@ -1272,7 +1283,7 @@ empathy_contact_list_view_new (EmpathyContactListStore        *store,
 			       EmpathyContactFeatureFlags      contact_features)
 {
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store), NULL);
-	
+
 	return g_object_new (EMPATHY_TYPE_CONTACT_LIST_VIEW,
 			     "store", store,
 			     "contact-features", contact_features,
@@ -1305,6 +1316,31 @@ empathy_contact_list_view_dup_selected (EmpathyContactListView *view)
 	return contact;
 }
 
+EmpathyContactListFlags
+empathy_contact_list_view_get_flags (EmpathyContactListView *view)
+{
+	EmpathyContactListViewPriv *priv;
+	GtkTreeSelection          *selection;
+	GtkTreeIter                iter;
+	GtkTreeModel              *model;
+	EmpathyContactListFlags    flags;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), 0);
+
+	priv = GET_PRIV (view);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		return 0;
+	}
+
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, &flags,
+			    -1);
+
+	return flags;
+}
+
 gchar *
 empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 {
@@ -1344,7 +1380,7 @@ contact_list_view_remove_dialog_show (GtkWindow   *parent,
 {
 	GtkWidget *dialog;
 	gboolean res;
-	
+
 	dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
 					 "%s", message);
@@ -1446,7 +1482,7 @@ contact_list_view_remove_activate_cb (GtkMenuItem            *menuitem,
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	EmpathyContact             *contact;
-		
+
 	contact = empathy_contact_list_view_dup_selected (view);
 
 	if (contact) {
@@ -1455,7 +1491,7 @@ contact_list_view_remove_activate_cb (GtkMenuItem            *menuitem,
 
 		parent = empathy_get_toplevel_window (GTK_WIDGET (view));
 		text = g_strdup_printf (_("Do you really want to remove the contact '%s'?"),
-					empathy_contact_get_name (contact));						
+					empathy_contact_get_name (contact));
 		if (contact_list_view_remove_dialog_show (parent, _("Removing contact"), text)) {
 			EmpathyContactList *list;
 
@@ -1476,6 +1512,7 @@ empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view)
 	GtkWidget                  *menu;
 	GtkWidget                  *item;
 	GtkWidget                  *image;
+	EmpathyContactListFlags     flags;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view), NULL);
 
@@ -1483,25 +1520,23 @@ empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view)
 	if (!contact) {
 		return NULL;
 	}
+	flags = empathy_contact_list_view_get_flags (view);
 
 	menu = empathy_contact_menu_new (contact, priv->contact_features);
 
-	if (!(priv->list_features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE)) {
-		g_object_unref (contact);
-		return menu;
-	}
-
-	if (menu) {
-		/* Separator */
-		item = gtk_separator_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
-	} else {
-		menu = gtk_menu_new ();
-	}
-
 	/* Remove contact */
-	if (priv->list_features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE) {
+	if (priv->list_features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE &&
+	    flags & EMPATHY_CONTACT_LIST_CAN_REMOVE) {
+		/* create the menu if required, or just add a separator */
+		if (!menu) {
+			menu = gtk_menu_new ();
+		} else {
+			item = gtk_separator_menu_item_new ();
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			gtk_widget_show (item);
+		}
+
+		/* Remove */
 		item = gtk_image_menu_item_new_with_mnemonic (_("_Remove"));
 		image = gtk_image_new_from_icon_name (GTK_STOCK_REMOVE,
 						      GTK_ICON_SIZE_MENU);
